
function [R,C] = Cumulant(cumstar,order,x,xtilde,z,D);

% [R,C] = Cumulant(cumstar,order,x,xtilde,z,D);
%
% Estimation of the covariances and cumulants for the optimization of the
%   contrast in [1]
% makeRC estimates the matrices R_x and C_{x,z} described in Eq. (38)
%   of [1]
%
% Inputs:
% -------
% cumstar = (4 x 1) vector, where the conjugate are in the C_{x,z}
% order = 2, 3 or 4 , if we have 2, 1 or no reference signals 
% x = (Nx x Nsample) matrix containing the Nx observations of the mixture
% xtilde = ((Nx x D) x Nsample) matrix containing the (Nx x D) delayed observations of the mixture
% z = (1 x Nsample) vector containing the reference signal
% D = length of the separating filter
% 
% Outputs: 
% -------
% R = (Nx x Nx) covariance matrix
% C = (Nx x Nx x Nx) cumulant matrix
%
% -------
% Author: Rémi Dubroca
% Created: 06/2010
% Last modified: 01/07/2010
% Contact: dubroca@univ-tln.fr
%
% Ref:
% [1] A General Algebraic Algorithm for Blind Extraction
% of one Source in a MIMO Convolutive Mixture / R. Dubroca, C. De Luigi, M. Castella and E. Moreau
% IEEE Trans. on Signal Processing 58(5), May 2010


if (size(x,2)-D+1)~=size(z,2),
    error('xtilde et z n''ont pas meme longueur');
else Ne = size(x,2);
end;


Nx = size(x,1);
Nxtilde=size(xtilde,1);
Nextilde=size(xtilde,2);

R = cov(xtilde');
zfilde=z;


switch order
    case 2
        Z = repmat(z,Nx*D,1);        
        Zxtilde1=conjcum(Z,cumstar(3)).*conjcum(xtilde,cumstar(1));
        Zxtilde2=conjcum(Z,cumstar(4)).*conjcum(xtilde,cumstar(2));
        
        C = 1/Ne*Zxtilde1*Zxtilde2.' ...    %E{xi,xj*,z*,z}
         -1/Ne^2*(conjcum(xtilde,cumstar(1))*conjcumt(xtilde,cumstar(2)))*(conjcum(zfilde,cumstar(4))*conjcumt(zfilde,cumstar(3))) ... %E{xi,xj*}*E{z*,z}
         -1/Ne^2*(conjcum(xtilde,cumstar(1))*conjcumt(zfilde,cumstar(3)))*(conjcum(xtilde,cumstar(2))*conjcumt(zfilde,cumstar(4))).' ... %E{xi,z*}*E{xj*,z}
         -1/Ne^2*(conjcum(xtilde,cumstar(1))*conjcumt(zfilde,cumstar(4)))*(conjcum(xtilde,cumstar(2))*conjcumt(zfilde,cumstar(3))).';  %E{xi,z}*E{xj*,z*}  
     
        clear Zxtilde1 Zxtilde2
        clear Z
    case 3
          Z = repmat(z,Nx*D,1);  
          repvec=kron(1:Nxtilde,ones(1,Nxtilde));
          Xxtilde = conjcum(xtilde(repvec,:),cumstar(2)).*repmat(conjcum(xtilde,cumstar(3)),Nxtilde,1); % [i,j*] ou [k,l*]* (Ns^2,Ne)
          
          C1 = 1/Ne*(conjcum(Z,cumstar(4)).*conjcum(xtilde,cumstar(1)))*(Xxtilde).' ...                               % E{xi,xj*,z*,xk}             
              -1/Ne^2*kron(conjcum(xtilde,cumstar(1))*conjcumt(xtilde,cumstar(2)),conjcum(zfilde,cumstar(4))*conjcumt(xtilde,cumstar(3)) )... % E{xi,xj*}*E{z*,xk}
              -1/Ne^2*kron((conjcum(zfilde,cumstar(4))*conjcumt(xtilde,cumstar(1))).',reshape((conjcum(xtilde,cumstar(2))*conjcumt(xtilde,cumstar(3))).',1,Nxtilde*Nxtilde)) ... % E{xi,z*}*E{xj*,xk}
              -1/Ne^2*kron(conjcum(zfilde,cumstar(4))*conjcumt(xtilde,cumstar(2)),conjcum(xtilde,cumstar(1))*conjcumt(xtilde,cumstar(3)));                   % E{xi,xk}*E{xj*,z*}
          C = folding(C1,1,[Nxtilde Nxtilde Nxtilde]); 

          clear Z repvec Xxtilde
         
          
    case 4  
          repvec1=kron(1:Nxtilde,ones(1,Nxtilde));
          Xxtilde12 = conjcum(xtilde(repvec1,:),cumstar(1)).*repmat(conjcum(xtilde,cumstar(2)),Nxtilde,1);  % [i,j*] ou [k,l*]* (Ns^2,Ne)
          Xxtilde34 = conjcum(xtilde(repvec1,:),cumstar(3)).*repmat(conjcum(xtilde,cumstar(4)),Nxtilde,1);
          
          repvec2=kron(ones(1,Nxtilde),1:Nxtilde);
          Xxtilde114=conjcum(xtilde,cumstar(1))*conjcumt(xtilde,cumstar(4));
          Xxtilde132=conjcum(xtilde,cumstar(3))*conjcumt(xtilde,cumstar(2));          
          Xxtilde114=Xxtilde114(repvec1,repvec2);  % [i,j] (Ns^2,Ns^2)     i 1111 2222, j 123 123     
          Xxtilde132=Xxtilde132(repvec1,repvec2);  % [i,j] (Ns^2,Ns^2)     i 1111 2222, j 123 123                 
          
          C1 =   1/Ne*Xxtilde12*Xxtilde34.' ...                              % [i,j*] [k,l*]* E{xi,xj*,xk*,xl}  
              -1/Ne^2*kron(reshape(conjcum(xtilde,cumstar(2))*conjcumt(xtilde,cumstar(1)),Nxtilde*Nxtilde,1),reshape(conjcum(xtilde,cumstar(4))*conjcumt(xtilde,cumstar(3)),1,Nxtilde*Nxtilde)) ... % [j*,i][l,k*] E{xi,xj*}*E{xk*,xl}
              -1/Ne^2*kron((conjcum(xtilde,cumstar(1))*conjcumt(xtilde,cumstar(3))),conjcum(xtilde,cumstar(2))*conjcumt(xtilde,cumstar(4))) ... % [i,k*] [j*,l)      E{xi,xk*}*E{xj*,xl}
              -1/Ne^2*Xxtilde114.*Xxtilde132.';                                                                               % [i,l] [k*,j*]      E{xi,xl}*E{xj*,xk*}
          
          C = square_folding(C1,4);     
          clear repvec1 repvec2 Xxtilde12 Xxtilde34 Xxtilde114 Xxtilde132 
end
clear zfilde



